==> access-0.3.0%2B1.tar.gz: lib/src/dbtool/dbtool_create.dart <==
//Copyright (C) 2014 Potix Corporation. All Rights Reserved.
//History: Thu, Jul 03, 2014  6:38:28 PM
// Author: tomyeh
part of access.dbtool;

///Creates the tables specified in version.
Future create(Connection conn,
    double version, Map<String, Map<String, SqlType>> tables,
    Map<String, IndexInfo> indexes, Map<String, RuleInfo> rules,
    String script) {
  final Set<String> tblsGened = new HashSet();
  final List<_DeferredRef> refsDeferred = [];

  return Future.forEach(tables.keys,
    (String otype) =>
      _createTable(conn, otype, tables[otype], tblsGened, refsDeferred))
  .then((_) => Future.forEach(refsDeferred,
    (_DeferredRef defRef) => defRef.create(conn)))
  .then((_) => Future.forEach(indexes.keys,
    (String name) => _createIndex(conn, name, indexes[name])))
  .then((_) => Future.forEach(rules.keys,
    (String name) => _createRule(conn, name, rules[name])))
  .then((_) => conn.execute(script));
}

Future _createTable(Connection conn, String otype, Map<String, SqlType> table,
      Set<String> tblsGened, List<_DeferredRef> refsDeferred) {
  print("Creating table: $otype...");
  tblsGened.add(otype);

  final List<String> query = ['create table "', otype, '" (\n'];
  _genCreateColumns(query, otype, table, tblsGened, refsDeferred, true);
  query.add(')');
  return conn.execute(query.join(''));
}

class _DeferredRef {
  final String otypePrimary;
  final String otypeForeign;
  final String columnForeign;
  final String cascade;

  _DeferredRef(this.otypePrimary, this.otypeForeign, this.columnForeign,
    this.cascade);

  Future create(Connection conn)
  => conn.execute(['alter table "', otypeForeign,
      '" add constraint "fk_', columnForeign,
      '" foreign key("', columnForeign, '") references "',
      otypePrimary, '"("', F_OID, '") ', cascade].join(''));
  Future drop(Connection conn)
  => conn.execute(['alter table "', otypeForeign,
      '" drop constraint "fk_', columnForeign, '"'].join(''));
}

bool _genCreateColumns(List<String> query, String otype,
      Map<String, SqlType> table, Set<String> tblsGened,
      List<_DeferredRef> refsDeferred, bool first) {
  for (final String col in table.keys) {
    if (col.startsWith(COPY)) {
      first = _genCreateColumns(query, otype,
          (table[col] as CopyType).source, tblsGened, refsDeferred, first);
      continue;
    }

    if (first) first = false;
    else query.add(',\n');

    final SqlType sqlType = table[col];
    query..add('"')..add(col)..add('" ');
    if (sqlType is ReferenceType) {
      final ReferenceType refType = sqlType;
      final bool deferred = !tblsGened.contains(refType.otype);
      query.add(refType.toSqlStringBy(deferred));

      if (deferred)
        refsDeferred.add(
          new _DeferredRef(refType.otype, otype, col, refType.cascade));
    } else {
      query.add(sqlType.toSqlString());
    }
  }
  return first;
}

Future _createIndex(Connection conn, String name, IndexInfo info) {
  print("Creating index: $name...");

  final List<String> query = ['create '];
  if (info.unique)
    query.add('unique ');
  query..add('index "')..add(name)..add('" on "')
    ..add(info.table)..add('"(');

  bool first = true;
  for (final String col in info.columns) {
    if (first) first = false;
    else query.add(',');
    query..add('"')..add(col)..add('"');
  }
  query.add(')');

  return conn.execute(query.join(''));
}

Future _createRule(Connection conn, String name, RuleInfo info) {
  print("Creating rule: $name...");
  return conn.execute('create rule "$name" as ${info.rule}');
}

==> access-1.5.0%2B1.tar.gz: lib/src/dbtool/dbtool_create.dart <==
//Copyright (C) 2014 Potix Corporation. All Rights Reserved.
//History: Thu, Jul 03, 2014  6:38:28 PM
// Author: tomyeh
part of access.dbtool;

///Creates the tables specified in version.
Future create(Connection conn,
    double version, Map<String, Map<String, SqlType>> tables,
    Map<String, IndexInfo> indexes, Map<String, RuleInfo> rules) async {
  final Set<String> tblsGened = HashSet<String>();
  final List<_DeferredRef> refsDeferred = [];

  for (final otype in tables.keys)
    await _createTable(conn, otype, tables[otype], tblsGened, refsDeferred);
  for (final defRef in refsDeferred)
    await defRef.create(conn);

  for (final name in indexes.keys)
    await _createIndex(conn, name, indexes[name]);
  for (final name in rules.keys)
    await _createRule(conn, name, rules[name]);
}

Future _createTable(Connection conn, String otype, Map<String, SqlType> table,
      Set<String> tblsGened, List<_DeferredRef> refsDeferred) {
  print("Creating table: $otype...");
  tblsGened.add(otype);

  final List<String> query = ['create table "', otype, '" (\n'];
  _genCreateColumns(query, otype, table, tblsGened, refsDeferred, true);
  query.add(')');
  return conn.execute(query.join(''));
}

class _DeferredRef {
  final String otypePrimary, columnPrimary;
  final String otypeForeign, columnForeign;
  final String cascade;

  _DeferredRef(this.otypePrimary, this.columnPrimary,
      this.otypeForeign, this.columnForeign, this.cascade);

  Future create(Connection conn)
  => conn.execute(['alter table "', otypeForeign,
      '" add constraint "fk_', columnForeign,
      '" foreign key("', columnForeign, '") references "',
      otypePrimary, '"("', columnPrimary, '") ', cascade].join(''));
  Future drop(Connection conn)
  => conn.execute(['alter table "', otypeForeign,
      '" drop constraint "fk_', columnForeign, '"'].join(''));
}

bool _genCreateColumns(List<String> query, String otype,
      Map<String, SqlType> table, Set<String> tblsGened,
      List<_DeferredRef> refsDeferred, bool first) {
  for (final String col in table.keys) {
    if (col.startsWith(copy)) {
      first = _genCreateColumns(query, otype,
          (table[col] as CopyType).source, tblsGened, refsDeferred, first);
      continue;
    }

    if (first) first = false;
    else query.add(',\n');

    final sqlType = table[col];

    if (col == primaryKey) {
      query..add('constraint "')..add(otype)..add('_pkey" primary key (');
      bool first = true;
      for (final col in (sqlType as PrimaryKeyType).columns) {
        if (first) first = false;
        else query.add(',');
        query..add('"')..add(col)..add('"');
      }
      query.add(')');
      continue;
    }

    if (col.startsWith(define)) {
      query.add(sqlType.toSqlString());
      continue;
    }

    query..add('"')..add(col)..add('" ');
    if (sqlType is ReferenceType) {
      final ReferenceType refType = sqlType;
      final bool deferred = !tblsGened.contains(refType.otype);
      query.add(refType.toSqlStringBy(deferred));

      if (deferred)
        refsDeferred.add(
          _DeferredRef(refType.otype, refType.column,
              otype, col, refType.cascade));
    } else {
      query.add(sqlType.toSqlString());
    }
  }
  return first;
}

Future _createIndex(Connection conn, String name, IndexInfo info) {
  print("Creating index: $name...");

  final List<String> query = ['create '];
  if (info.unique)
    query.add('unique ');
  query..add('index "')..add(name)..add('" on "')
    ..add(info.table)..add('"');

  if (info.using != null)
    query..add(' using ')..add(info.using)..add(' ');

  query.add('(');

  bool first = true;
  for (final String col in info.columns) {
    if (first) first = false;
    else query.add(',');
    query..add('"')..add(col)..add('"');
  }
  if (info.ops != null)
    query..add(' ')..add(info.ops);
  query.add(')');

  if (info.where != null)
    query..add(' where ')..add(info.where);

  return conn.execute(query.join(''));
}

Future _createRule(Connection conn, String name, RuleInfo info) {
  print("Creating rule: $name...");
  return conn.execute('create rule "$name" as ${info.rule}');
}
